/** 
	Programa para crear y visualizar
  un cilindro con cuadrilateros y un
  solo color por cara
	
	Fraga 10/10/2013
**/
#include <qpainter.h>
#include <ctype.h>
#include "cubo.h"

Cubo::Cubo( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer ), parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	crea_cilindro( cil1, 10, 2.5, 8.0 );
}


void Cubo::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cubo::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cubo::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cubo::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// En z estn en coordenadas negativas
	if( w <= h )
		glOrtho ( -10.0, 10.0, -10.0/aspect, 10.0/aspect, -10.0, 10.0);
	else
		glOrtho (-10.0*aspect, 10.0*aspect, -10.0, 10.0,  -10.0, 10.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Cubo::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cubo::crea_cilindro( float v[][3], int n, float r, float l )
{
	int i;
	float alfa;
	float s, c, ang;

	alfa = 2.0*3.141592654/n;

	for ( i=0; i<n; i++ ) {
		ang = alfa*i;
		s = r*sin( ang );
		c = r*cos( ang );
		v[2*i][0] = c;
		v[2*i][1] = -l/2.0;
		v[2*i][2] = s;

		v[2*i+1][0] = c;
		v[2*i+1][1] = l/2.0;
		v[2*i+1][2] = s;
	}
}

void Cubo::dibuja_cilindro( float v[][3],  int n )
{
	float color[2][3] = { {1.0,1.0,1.0}, {0.0,0.0,1.0} };
	int i,j;

	glBegin( GL_QUAD_STRIP );

	glVertex3fv( &v[ 0 ][0] );
	glVertex3fv( &v[ 1 ][0] );
	for( i=1; i<n; i++ ){
		j = 2*i;
		glVertex3fv( &v[j][0] );
		glColor3fv( &color[i%2][0] );
		glVertex3fv( &v[j+1][0] );
	}
	glVertex3fv( &v[ 0 ][0] );
	glColor3fv( &color[i%2][0] );
	glVertex3fv( &v[ 1 ][0] );

	glEnd();
}

void Cubo::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	dibuja_cilindro( cil1, 10 );

	glPopMatrix( );

	swapBuffers( );
}

void Cubo::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);

	glShadeModel( GL_FLAT );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}


